==> krb4-0.9.tar.gz: krb4-0.9/lib/krb/mk_priv.c <==
#include "krb_locl.h"

RCSID("$Id: mk_priv.c,v 1.10 1996/05/15 03:39:47 joda Exp $");

/* application include files */
#include "lsb_addr_comp.h"

/*
 * krb_mk_priv() constructs an AUTH_MSG_PRIVATE message.  It takes
 * some user data "in" of "length" bytes and creates a packet in "out"
 * consisting of the user data, a timestamp, and the sender's network
 * address.
 * The packet is encrypted by pcbc_encrypt(), using the given
 * "key" and "schedule".
 * The length of the resulting packet "out" is
 * returned.
 *
 * It is similar to krb_mk_safe() except for the additional key
 * schedule argument "schedule" and the fact that the data is encrypted
 * rather than appended with a checksum.  Also, the protocol version
 * number is "private_msg_ver", defined in krb_rd_priv.c, rather than
 * KRB_PROT_VERSION, defined in "krb.h".
 *
 * The "out" packet consists of:
 *
 * Size			Variable		Field
 * ----			--------		-----
 *
 * 1 byte		private_msg_ver		protocol version number
 * 1 byte		AUTH_MSG_PRIVATE |	message type plus local
 *			HOST_BYTE_ORDER		byte order in low bit
 *
 * 4 bytes		c_length		length of data
 * we encrypt from here with pcbc_encrypt
 * 
 * 4 bytes		length			length of user data
 * length		in			user data
 * 1 byte		msg_time_5ms		timestamp milliseconds
 * 4 bytes		sender->sin.addr.s_addr	sender's IP address
 *
 * 4 bytes		msg_time_sec or		timestamp seconds with
 *			-msg_time_sec		direction in sign bit
 *
 * 0<=n<=7  bytes	pad to 8 byte multiple	zeroes
 */

int32_t
krb_mk_priv(void *in, void *out, u_int32_t length, 
	    struct des_ks_struct *schedule, des_cblock *key, 
	    struct sockaddr_in *sender, struct sockaddr_in *receiver)
{
    unsigned char *p = (unsigned char*)out;
    unsigned char *cipher;

    struct timeval tv;
    int32_t msg_time_sec;

    u_int32_t src_addr;

    u_int32_t len;

    p += put_int(KRB_PROT_VERSION, p, 1);
    p += put_int(AUTH_MSG_PRIVATE, p, 1);

    len = 4 + length + 1 + 4 + 4;
    len = (len + 7) & ~7;
    p += put_int(len, p, 4);
    
    cipher = p;

    p += put_int(length, p, 4);
    
    memcpy(p, in, length);
    p += length;
    
    gettimeofday(&tv, NULL);

    *p++ =tv.tv_usec / 5000;
    
    /* see comment in rd_safe */
    src_addr = htonl(sender->sin_addr.s_addr);
    p += put_int(src_addr, p, 4);

    p += put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4);
    
    memset(p, 0, 7);

    des_pcbc_encrypt((des_cblock *)cipher, (des_cblock *)cipher,
		     len, schedule, key, DES_ENCRYPT);

    return  (cipher - (unsigned char*)out) + len;
}

==> krb4-0.9.1.tar.gz: krb4-0.9.1/lib/krb/mk_priv.c <==
#include "krb_locl.h"

RCSID("$Id: mk_priv.c,v 1.12 1996/06/25 13:34:03 joda Exp $");

/* application include files */
#include "lsb_addr_comp.h"

/*
 * krb_mk_priv() constructs an AUTH_MSG_PRIVATE message.  It takes
 * some user data "in" of "length" bytes and creates a packet in "out"
 * consisting of the user data, a timestamp, and the sender's network
 * address.
 * The packet is encrypted by pcbc_encrypt(), using the given
 * "key" and "schedule".
 * The length of the resulting packet "out" is
 * returned.
 *
 * It is similar to krb_mk_safe() except for the additional key
 * schedule argument "schedule" and the fact that the data is encrypted
 * rather than appended with a checksum.  Also, the protocol version
 * number is "private_msg_ver", defined in krb_rd_priv.c, rather than
 * KRB_PROT_VERSION, defined in "krb.h".
 *
 * The "out" packet consists of:
 *
 * Size			Variable		Field
 * ----			--------		-----
 *
 * 1 byte		private_msg_ver		protocol version number
 * 1 byte		AUTH_MSG_PRIVATE |	message type plus local
 *			HOST_BYTE_ORDER		byte order in low bit
 *
 * 4 bytes		c_length		length of data
 * we encrypt from here with pcbc_encrypt
 * 
 * 4 bytes		length			length of user data
 * length		in			user data
 * 1 byte		msg_time_5ms		timestamp milliseconds
 * 4 bytes		sender->sin.addr.s_addr	sender's IP address
 *
 * 4 bytes		msg_time_sec or		timestamp seconds with
 *			-msg_time_sec		direction in sign bit
 *
 * 0<=n<=7  bytes	pad to 8 byte multiple	zeroes
 */

int32_t
krb_mk_priv(void *in, void *out, u_int32_t length, 
	    struct des_ks_struct *schedule, des_cblock *key, 
	    struct sockaddr_in *sender, struct sockaddr_in *receiver)
{
    unsigned char *p = (unsigned char*)out;
    unsigned char *cipher;

    struct timeval tv;
    int32_t msg_time_sec;

    u_int32_t src_addr;

    u_int32_t len;

    p += put_int(KRB_PROT_VERSION, p, 1);
    p += put_int(AUTH_MSG_PRIVATE, p, 1);

    len = 4 + length + 1 + 4 + 4;
    len = (len + 7) & ~7;
    p += put_int(len, p, 4);
    
    cipher = p;

    p += put_int(length, p, 4);
    
    memcpy(p, in, length);
    p += length;
    
    gettimeofday(&tv, NULL);

    *p++ =tv.tv_usec / 5000;
    
    src_addr = sender->sin_addr.s_addr;
    p += put_address(src_addr, p);

    p += put_int(lsb_time(tv.tv_sec, sender, receiver), p, 4);
    
    memset(p, 0, 7);

    des_pcbc_encrypt((des_cblock *)cipher, (des_cblock *)cipher,
		     len, schedule, key, DES_ENCRYPT);

    return  (cipher - (unsigned char*)out) + len;
}
